@@ -22,6 +22,7 @@ package File::StripNondeterminism::handlers::png;
 use strict;
 use warnings;
 
+use File::StripNondeterminism::Common qw(copy_data);
 use File::Basename qw/dirname/;
 use POSIX qw/strftime/;
 use List::Util qw/min/;
@@ -39,7 +40,8 @@ sub chunk {
 sub time_chunk {
 	my ($seconds) = @_;
 	my ($sec, $min, $hour, $mday, $mon, $year) = gmtime($seconds);
-	return chunk('tIME', pack('nCCCCC', 1900+$year, $mon+1, $mday, $hour, $min, $sec));
+	return chunk('tIME',
+		pack('nCCCCC', 1900+$year, $mon+1, $mday, $hour, $min, $sec));
 }
 
 sub text_chunk {
@@ -55,13 +57,14 @@ sub normalize {
 	open(my $fh, '+<', $filename) or die "$filename: open: $!";
 
 	if (_normalize($filename, $fh, $tempfile)) {
-		chmod((stat($fh))[2] & 07777, $tempfile->filename);
-		rename($tempfile->filename, $filename)
-			or die "$filename: unable to overwrite: rename: $!";
-		$tempfile->unlink_on_destroy(0);
+		$tempfile->close;
+		copy_data($tempfile->filename, $filename)
+		  or die "$filename: unable to overwrite: copy_data: $!";
 	}
 
 	close $fh;
+
+	return 1;
 }
 
 sub _normalize {
@@ -73,8 +76,9 @@ sub _normalize {
 	my $modified = 0;
 	my $bytes_read;
 
-	read($fh, my $magic, 8); $magic eq "\x89PNG\r\n\x1a\n"
-		or die "$filename: does not appear to be a PNG";
+	read($fh, my $magic, 8);
+	$magic eq "\x89PNG\r\n\x1a\n"
+	  or die "$filename: does not appear to be a PNG";
 
 	print $tempfile $magic;
 
@@ -96,12 +100,17 @@ sub _normalize {
 			}
 
 			if ($type eq "tIME") {
-				print $tempfile time_chunk($canonical_time) if defined($canonical_time);
+				print $tempfile time_chunk($canonical_time)
+				  if defined($canonical_time);
 				$modified = 1;
 				next;
-			} elsif (($type =~ /[tiz]EXt/) && ($data =~ /^(date:[^\0]+|Creation Time)\0/)) {
-				print $tempfile text_chunk($1, strftime("%Y-%m-%dT%H:%M:%S-00:00",
-								gmtime($canonical_time))) if defined($canonical_time);
+			} elsif (($type =~ /[tiz]EXt/)
+				&& ($data =~ /^(date:[^\0]+|Creation Time)\0/)) {
+				print $tempfile text_chunk(
+					$1,
+					strftime(
+						"%Y-%m-%dT%H:%M:%S-00:00",gmtime($canonical_time))
+				) if defined($canonical_time);
 				$modified = 1;
 				next;
 			}
@@ -141,7 +150,7 @@ sub _normalize {
 	}
 	defined($bytes_read) or die "$filename: read failed: $!";
 	warn "$filename: $garbage bytes of garbage after IEND chunk"
-		if $garbage > 0;
+	  if $garbage > 0;
 
 	return $modified;
 }
